@@ -1,5 +1,5 @@
 from collections import defaultdict, Counter
-from typing import List, Dict, Set, Tuple
+from typing import List, Dict, Tuple
 import itertools
 import mlflow
 import os
@@ -18,17 +18,19 @@ from databricks.feature_store.entities.feature_tables_for_serving import (
 )
 from databricks.feature_store.entities.online_feature_table import (
     AbstractOnlineFeatureTable,
-    OnlineFeatureTable,
     PrimaryKeyDetails,
 )
-
 from databricks.feature_store.online_lookup_client import OnlineLookupClient
-
 from databricks.feature_store import mlflow_model_constants
 from databricks.feature_store.feature_lookup_version import VERSION
 
 # The provisioner of this model is expected to set an environment variable with the path to a
 # feature_tables_for_serving.dat file.
+from databricks.feature_store.utils.uc_utils import (
+    get_feature_spec_with_full_table_names,
+    get_feature_spec_with_reformat_full_table_names,
+)
+
 FEATURE_TABLES_FOR_SERVING_FILEPATH_ENV = "FEATURE_TABLES_FOR_SERVING_FILEPATH"
 
 LookupKeyType = Tuple[str, ...]
@@ -65,7 +67,18 @@ class _FeatureStoreModelWrapper:
 
         raw_model_path = os.path.join(path, mlflow_model_constants.RAW_MODEL_FOLDER)
         self.raw_model = mlflow.pyfunc.load_model(raw_model_path)
-        self.feature_spec = FeatureSpec.load(path)
+
+        # Reformat local metastore 3L tables to 2L. Non-local metastore 3L tables and 2L tables are unchanged.
+        # This guarantees table name consistency between feature_spec and feature_tables_for_serving.
+        # https://docs.google.com/document/d/1x_V9GshlnoAAFFCuDsXWdJVtop9MG2HWTUo5IK_1mEw
+        original_feature_spec = FeatureSpec.load(path)
+        # We call get_feature_spec_with_full_table_names to append the default metastore to 2L names,
+        # as get_feature_spec_with_reformat_full_table_names expects full 3L table names and throws otherwise.
+        feature_spec_3l = get_feature_spec_with_full_table_names(original_feature_spec)
+        self.feature_spec = get_feature_spec_with_reformat_full_table_names(
+            feature_spec_3l
+        )
+
         self.ft_metadata = self._get_ft_metadata(
             self.feature_spec, feature_tables_for_serving
         )
